require "camera"
require "stars"
require "enemy"
require "ship"
require "bullet"
require "entities.asteroid"

bodyQueu = {}
function bodyQueu.add(givenBody)
  if bodyID == nil then bodyID = 1 end
  bodyQueu[bodyID] = {}
  bodyQueu[bodyID].body = givenBody
  bodyQueu[bodyID].tick = bodyQueu.tick
  bodyID = bodyID + 1
end

function bodyQueu.update(dt)
  if(bodyQueu.tick == 90) then
    print("Queu wordt geleegd op tick:" .. bodyQueu.tick .. " met " .. #bodyQueu .. " en " .. world:getBodyCount( ) .. " World-objecten voor het legen")
  end
  
  if(#bodyQueu ~= nil) then
    for bodyID = 1, #bodyQueu do
      if(bodyQueu[bodyID].tick ~= bodyQueu.tick) then
        print(bodyQueu[bodyID].tick .. " vs " .. bodyQueu.tick .. " ID:" .. bodyID)
        
        
        if(bodyQueu[bodyID] ~= nil and world:isLocked() == false) then
          bodyQueu[bodyID].body:destroy()
          table.remove(bodyQueu, bodyID)
        end -- ??
        
                
      end
    end
  end
    
  if(bodyQueu.tick == 90) then 
    print(#bodyQueu .. " Tabel-objecten en " .. world:getBodyCount( ) .. " World-objecten na legen")
  end
  
  if(bodyQueu.tick >= 100) then
    bodyQueu.tick = 0
  else
    bodyQueu.tick = bodyQueu.tick + 1
  end
  
  world:update(dt)
end

function love.load()
  --Graphical Display
  info = false;
  stars_load(125)   
	medium = love.graphics.newFont(80)
	love.graphics.setBackgroundColor(255,255,255)
  
  --Physics
  love.physics.setMeter(65)
  world = love.physics.newWorld(0,0, true)
  world:setCallbacks(beginContact, endContact, preSolve, postSolve)
  bodyQueu.tick = 0
  
  --Entities
  ship.load()
  asteroid.spawn(200,300)
  asteroid.spawn(0,0)
end

 function love.update(dt) 
  --Dont ever ever go faster then 60 calculations per second NOTE: Not frames per second!
  local MAX_DT = 1/60;
  if(dt > MAX_DT) then    
    dt = MAX_DT
  end
    
  --Updates the physics world
  bodyQueu.update(dt)
  --world:update(dt)
  ship.move()
    
  --Random updates
  stars_update(dt)
  asteroid.update()

  --Updating the camera, currently stuck to Player
  camera:setPosition( ship.body:getX() - ((love.graphics.getWidth()/2)*camera.scaleX),
                      ship.body:getY() - ((love.graphics.getHeight()/2)*camera.scaleY)  )
end


--[CallBacks]
--[beginContact] gets called when two fixtures start overlapping (two objects collide).
--[endContact] gets called when two fixtures stop overlapping (two objects disconnect).
--[preSolve] is called just before a frame is resolved for a current collision
--[postSolve] is called just after a frame is resolved for a current collision
function beginContact(a, b, coll)
  asteroid.Collision(a,b)
  -- x,y = coll:getNormal() --X,Y Coordinates when touching
  --text = text.."\n"..a:getUserData().." colliding with "..b:getUserData().." with a vector normal of: "..x..", "..y
end

function endContact(a, b, coll)
end

function preSolve(a, b, coll)
end

function postSolve(a, b, coll)
end


--Drawing the game
function love.draw()
  --Background is static and the stars have been coded manually
  bg_draw()
  stars_draw()
  
  --Everything between the camera tags is automaticly adjusted
  camera:set()
    ship.draw()
    bullet.draw()
    asteroid.draw()
  camera:unset()
    
  --Mouse Rot
  local mouseX = love.mouse.getX()
  local mouseY = love.mouse.getY()
  --For some reason it refused to do getWidth()/2 | getHeight()/2
  local rot = math.atan2(mouseY - 384, mouseX - 540) 
  
  --Some print on screen with details about the game
  love.graphics.setColor(255,255,255)
  if(info == true) then
    love.graphics.print("Rotatie " .. ship.body:getAngle() ..
                        "\nX-Axis: " .. ship.body:getX() ..
                        "\nY-Axis: " .. ship.body:getY() ..
                        "\nSpeed: " .. ship.body:getLinearVelocity() ..
                        "\nScreen Width: " .. love.graphics.getWidth() ..
                        "\nScreen Height: " .. love.graphics.getHeight() ..
                        "\nCamera.X: " .. camera.x ..
                        "\nCamera.Y: " .. camera.y ..
                        "\nMouse.X: " .. love.mouse.getX() ..
                        "\nMouse.Y: " .. love.mouse.getY() ..
                        "\nMouse.ROT: " .. rot ..
                        "\nTick Queu: " .. bodyQueu.tick ..
                        "\nIn Queu: " .. #bodyQueu ..
                        "\nZoom Factor: " .. camera.scaleX
                        , 0 , 0)
    love.graphics.print("\nAsteroid1 HP: " .. asteroid[1].health ..
                        "\nAsteroid2 HP: " .. asteroid[2].health
                        , 2, love.graphics.getHeight()-45)
  end
  love.graphics.print("FPS: "..tostring(love.timer.getFPS( )), 850,0)
end

--Control Input
function love.keypressed(key)
  if(key == "i") then
    if(info == true) then info = false else info = true end
  end
end

function love.mousepressed(x,y, button)
  camera:zoom(button)
  bullet.shoot(button)
end